﻿using GalaSoft.MvvmLight;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.WeimoCube.Model
{
  /// <summary>
  /// 主窗口实体模型
  /// </summary>
  public class MainWModel : ObservableObject
  {
    public event PropertyChangedEventHandler? PropertyChanged;
    private string _ip;
    /// <summary>
    /// ip地址
    /// </summary>
    public string Ip
    {
      get { return _ip; }
      set
      {
        Set<string>(ref _ip, value);
      }
    }
    private string _port;
    /// <summary>
    /// 端口
    /// </summary>
    public string Port
    {
      get { return _port; }
      set { Set<string>(ref _port, value); }
    }

    private string _RecipeNumberAddress;
    /// <summary>
    /// 配方编号地址
    /// </summary>
    public string RecipeNumberAddress { get { return _RecipeNumberAddress; } set { Set<string>(ref _RecipeNumberAddress, value); } }

    private string _DisplayOrderNumberAddress;
    /// <summary>
    /// 显示订单编号地址
    /// </summary>
    public string DisplayOrderNumberAddress { get { return _DisplayOrderNumberAddress; } set { Set<string>(ref _DisplayOrderNumberAddress, value); } }

    private string _WriteNumberAddress;
    /// <summary>
    /// 写入编号地址
    /// </summary>
    public string WriteNumberAddress { get { return _WriteNumberAddress; } set { Set<string>(ref _WriteNumberAddress, value); } }

    private string _StartIngredientAddress;
    /// <summary>
    /// 启动配料地址
    /// </summary>
    public string StartIngredientAddress { get { return _StartIngredientAddress; } set { Set<string>(ref _StartIngredientAddress, value); } }

    private string _IngredientCompletionAddress;
    /// <summary>
    /// 配料完成地址
    /// </summary>
    public string IngredientCompletionAddress { get { return _IngredientCompletionAddress; } set { Set<string>(ref _IngredientCompletionAddress, value); } }

    private string _DeviceStatusAddress;
    /// <summary>
    /// 设备状态地址
    /// </summary>
    public string DeviceStatusAddress { get { return _DeviceStatusAddress; } set { Set<string>(ref _DeviceStatusAddress, value); } }

    private string _InformationErrorAddress;
    /// <summary>
    /// 信息错误地址
    /// </summary>
    public string InformationErrorAddress { get { return _InformationErrorAddress; } set { Set<string>(ref _InformationErrorAddress, value); } }



    private bool _TCPCRWIsEnabled = true;
    /// <summary>
    /// 连接ModbusTcp是否启用
    /// </summary>
    public bool TCPCRWIsEnabled
    {
      get { return _TCPCRWIsEnabled; }
      set { Set<bool>(ref _TCPCRWIsEnabled, value); }
    }

    private bool _TCPDRWIsEnabled = false;
    /// <summary>
    /// 断开连接按钮是否启用
    /// </summary>
    public bool TCPDRWIsEnabled
    {
      get { return _TCPDRWIsEnabled; }
      set { Set<bool>(ref _TCPDRWIsEnabled, value); }
    }


    private bool _isStart =true;
    /// <summary>
    /// 是否监控
    /// </summary>
    public bool IsStart
    {
      get { return _isStart; }
      set
      {
        Set<bool>(ref _isStart, value);
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("IsStart"));
        if (!value)
        {
          this.ShowMessage("监控未开始，请打开监控开关",false);
          this.ConnectionStatus = value;
        }

      }
    }

    private bool _ConnectionStatus;
    /// <summary>
    /// 连接状态
    /// </summary>
    public bool ConnectionStatus
    {
      get { return _ConnectionStatus; }
      set { Set<bool>(ref _ConnectionStatus, value); }
    }

    private int _WriteDataLength;
    /// <summary>
    /// 写入数据长度
    /// </summary>
    public int WriteDataLength
    {
      get { return _WriteDataLength; }
      set {Set<int>(ref _WriteDataLength , value); }
    }

    private ushort _ReadDataLength;
    /// <summary>
    /// 读取数据长度
    /// </summary>
    public ushort ReadDataLength
    {
      get { return _ReadDataLength; }
      set {Set<ushort>(ref _ReadDataLength , value); }
    }



    private object _CurrentOrderNumber;
    /// <summary>
    /// 当前订单编号
    /// 可以用于显示当前的订单编号
    /// </summary>
    public object CurrentOrderNumber
    {
      get { return _CurrentOrderNumber; }
      set { Set(ref _CurrentOrderNumber, value); }
    }


    private object _FormulaNumber ;
    /// <summary>
    /// 配方编号
    /// 需要调用的配方编号(编号来源见附件)
    /// </summary>
    public object FormulaNumber
    {
      get { return _FormulaNumber; }
      set { Set(ref _FormulaNumber, value); }
    }

    private bool _DetermineWriteNumber;
    /// <summary>
    /// 确定写入编号
    /// 确认配方数据,并下发到配料机
    /// </summary>
    public bool DetermineWriteNumber
    {
      get { return _DetermineWriteNumber; }
      set { Set(ref _DetermineWriteNumber, value); }
    }

    private bool _StartingIngredients;
    /// <summary>
    /// 启动配料
    /// 根据当前选择配方启动配料(当前模式不需要使用该功能，预留)
    /// </summary>
    public bool StartingIngredients
    {
      get { return _StartingIngredients; }
      set { Set(ref _StartingIngredients, value); }
    }

    private bool _IngredientsCompleted;
    /// <summary>
    /// 配料完成
    /// 启动配料后,待动作完成该值为True,再次执行配料请求为False
    /// </summary>
    public bool IngredientsCompleted
    {
      get { return _IngredientsCompleted; }
      set { Set(ref _IngredientsCompleted, value); }
    }

    private bool _DeviceStatus;
    /// <summary>
    /// 设备状态
    /// 上电为False,推送了订单为True
    /// </summary>
    public bool DeviceStatus
    {
      get { return _DeviceStatus; }
      set { Set(ref _DeviceStatus, value); }
    }

    private string _TopDianState;
    /// <summary>
    /// 设备上电状态显示
    /// </summary>
    public string TopDianState
    {
      get { return _TopDianState; }
      set { Set(ref _TopDianState, value); }
    }

    private string _OrderPushState;
    /// <summary>
    /// 订单推送状态显示
    /// </summary>
    public string OrderPushState
    {
      get { return _OrderPushState; }
      set { Set(ref _OrderPushState, value); }
    }

    private bool _InformationError;
    /// <summary>
    /// 信息错误   
    /// 点击了取消配料,该标志位为True.上位机收到信息后主动置False
    /// </summary>
    public bool InformationError
    {
      get { return _InformationError; }
      set { Set(ref _InformationError, value); }
    }


    private bool _isLog;
    /// <summary>
    /// 打开日志
    /// </summary>
    public bool IsLog
    {
      get { return _isLog; }
      set
      {
        Set<bool>(ref _isLog, value);
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("IsLog"));
      }
    }

    private int _blurRadius = 0;
    /// <summary>
    /// 主界面的 模糊半径
    /// </summary>
    public int BlurRadius
    {
      get { return _blurRadius; }
      set
      {

        Set<int>(ref _blurRadius, value);
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("BlurRadius"));
      }
    }

    private DateTime _currentDate;
    /// <summary>
    /// 当前时间
    /// </summary>
    public DateTime CurrentDate
    {
      get { return _currentDate; }
      set
      {
        //_currentDate = value;
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("CurrentDate"));
        Set<DateTime>(ref _currentDate, value);

      }
    }


    private string _message;
    /// <summary>
    /// 错误信息
    /// </summary>
    public string Message
    {
      get { return _message; }
      set
      {
        //_message = value;
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("Message"));
        Set<string>(ref _message, value);

      }
    }
    private string _msgColor = "#90EE90";
    /// <summary>
    /// 错误信息颜色
    /// </summary>
    public string MsgColor
    {
      get { return _msgColor; }
      set
      {
        //_msgColor = value;
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("MsgColor"));
        Set<string>(ref _msgColor, value);

      }
    }

    private string _logPath;
    /// <summary>
    /// 保存日志路径
    /// </summary>
    public string LogPath
    {
      get { return _logPath; }
      set
      {
        Set<string>(ref _logPath, value);
        //PropertyChanged?.Invoke(this, new PropertyChangedEventArgs("LogPath"));
      }
    }

    public void ShowMessage(string message, bool state = true)
    {
      this.Message = message;
      this.MsgColor = state ? "#90EE90" : "#FF4500";
    }

  }
}
