﻿using BPA.Helper;
using BPA.WeimoCube.Common;
using BPA.WeimoCube.Model;
using CommcationLibray;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Microsoft.Win32;
using S7.Net.Types;
using SharpCompress.Common;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using ThingsGateway.Foundation.Core;
using ThingsGateway.Foundation.Extension;

namespace BPA.WeimoCube.ViewModels
{
  /// <summary>
  /// 主实体视图模型
  /// </summary>
  public class MainWindowViewModel : ObservableObject
  {
    #region 属性
    //配方附件
    public ObservableCollection<FormulaAttachment> formulaAttachment { get; set; } = new ObservableCollection<FormulaAttachment>();
    /// <summary>
    /// 数据类型
    /// </summary>
    public ObservableCollection<RecipeNumberDataType> dataTypes { get; set; } = new ObservableCollection<RecipeNumberDataType>();
    /// <summary>
    /// 数据格式
    /// </summary>
    public ObservableCollection<FormulaNumberDataFormat> formulas { get; set; } = new ObservableCollection<FormulaNumberDataFormat>();

    private List<Task> tasks = new List<Task>();
    private CancellationTokenSource cts = new CancellationTokenSource();
    private ModbusTCP modbus = new ModbusTCP();
    /// <summary>
    /// 实例主窗口实体类
    /// </summary>
    public MainWModel MainWModel { get; set; } = new MainWModel();
    /// <summary>
    /// 日志操作类
    /// </summary>
    private LogProvider LogProvider;

    /// <summary>
    /// 连接ModbusTcp 事件
    /// </summary>
    public RelayCommand<object> ConnectMTCPCommand { get; set; }
    /// <summary>
    /// 定义设置事件
    /// </summary>
    public RelayCommand<object> ConfigCommand { get; set; }
    /// <summary>
    /// 定义关闭设置配置事件
    /// </summary>
    public RelayCommand<object> ConfigCloseCommand { get; set; }
    /// <summary>
    /// 定义开打文件夹事件
    /// </summary>
    public RelayCommand<object> LogPathSelectCommand { get; set; }
    /// <summary>
    /// 打开日志事件
    /// </summary>
    public RelayCommand<object> OpenLogPathCommand { get; set; }
    /// <summary>
    /// 定义写入编号事件
    /// </summary>
    public RelayCommand<object> WriteNumberCommand { get; set; }
    /// <summary>
    /// 定义确定写入编号事件
    /// </summary>
    public RelayCommand<object> DetermineWriteNumberCommand { get; set; }
    /// <summary>
    /// 定义启动配料事件
    /// </summary>
    public RelayCommand<object> StartingIngredientsCommand { get; set; }
    /// <summary>
    /// 定义打开配料附件事件
    /// </summary>
    public RelayCommand<object> OpenIngredientAttachmentCommand { get; set; }
    /// <summary>
    /// 附件列表行点击事件
    /// </summary>
    public RelayCommand<object> DoubleClickCommand { get; set; }
    /// <summary>
    /// 定义退出事件
    /// </summary>
    public RelayCommand<object> CloseCommand { get; set; }

    /// <summary>
    /// 订单推送 标志位
    /// 0表示已推送
    /// 1表示已完成
    /// </summary>
    private int OrderPushStatus = 0;
    /// <summary>
    /// 确定写入编号的记录 ，如果当前订单还没操作   不能进行第二次写入
    /// </summary>
    private bool DetermineWriteStatus = false;
    /// <summary>
    /// 设备状态 为false  可以继续派发下一个订单
    /// </summary>
    private bool DeviceStatus;
    /// <summary>
    /// 是否取消订单
    /// </summary>
    private bool WhetherCancel;

    private RecipeNumberDataType _dataType;
    /// <summary>
    /// 数据类型下拉值初始显示
    /// </summary>
    public RecipeNumberDataType DataType
    {
      get => _dataType;
      set { Set<RecipeNumberDataType>(ref _dataType, value); }
    }

    private FormulaNumberDataFormat _dataFormulas;
    /// <summary>
    /// 数据格式下拉值初始显示
    /// </summary>
    public FormulaNumberDataFormat DataFormulas
    {
      get { return _dataFormulas; }
      set { Set<FormulaNumberDataFormat>(ref _dataFormulas, value); }
    }

    private RecipeNumberDataType _OrderNumberType;
    /// <summary>
    /// 数据类型下拉值初始显示
    /// </summary>
    public RecipeNumberDataType OrderNumberType
    {
      get => _OrderNumberType;
      set { Set<RecipeNumberDataType>(ref _OrderNumberType, value); }
    }

    private FormulaNumberDataFormat _OrderNumberData;
    /// <summary>
    /// 数据格式下拉值初始显示
    /// </summary>
    public FormulaNumberDataFormat OrderNumberData
    {
      get { return _OrderNumberData; }
      set { Set<FormulaNumberDataFormat>(ref _OrderNumberData, value); }
    }


    #endregion

    public MainWindowViewModel()
    {
      var task = Task.Run(async () =>
      {
        while (!cts.IsCancellationRequested)
        {
          MainWModel.CurrentDate = System.DateTime.Now;
          await Task.Delay(new TimeSpan(1, 0, 0));
        }
      }, cts.Token);
      tasks.Add(task);
      LogProvider = new LogProvider(MainWModel.LogPath);

      this.RaisePropertyChanged("MainWModel");
      ConnectMTCPCommand = new RelayCommand<object>(new Action<object>(DoConnectModbusTcpCommand));
      ConfigCommand = new RelayCommand<object>(new Action<object>(Config));
      ConfigCloseCommand = new RelayCommand<object>(new Action<object>(ConfigClose));
      LogPathSelectCommand = new RelayCommand<object>(new Action<object>(LogPathSelect));
      OpenLogPathCommand = new RelayCommand<object>(new Action<object>(OpenLogFolder));
      WriteNumberCommand = new RelayCommand<object>(new Action<object>(WriteNumber));

      DetermineWriteNumberCommand = new RelayCommand<object>(new Action<object>(DetermineWriteNumber));
      StartingIngredientsCommand = new RelayCommand<object>(new Action<object>(StartingIngredients));
      OpenIngredientAttachmentCommand = new RelayCommand<object>(new Action<object>(OpenIngredientAttachment));

      DoubleClickCommand = new RelayCommand<object>(param => RowClick(param), param => true);

      CloseCommand = new RelayCommand<object>(new Action<object>(Close));

      this.SetRecipeNumberTypeData();
      this.SetFormulaNumberDataFormat();
      this.ChuShiHuaConfigData();
      this.GetFormulaData();
      this.StartMointor();
      //MainWModel.IsStart = true;
    }
    /// <summary>
    /// 附件列表行点击事件
    /// </summary>
    /// <param name="parameter"></param>
    private void RowClick(object parameter)
    {
      var selectedItem = (FormulaAttachment)parameter;
      try
      {
        MainWModel.FormulaNumber = (short)Convert.ToInt32(selectedItem.FormulaNumber);
      }
      catch (Exception ex)
      {
        MessageError($"配方编号类型转换失败!!! 失败信息：{ex.Message} + {selectedItem.ToString()}");
      }

    }

    /// <summary>
    /// 初始化配置文件数据
    /// </summary>
    private void ChuShiHuaConfigData()
    {
      // 配置数据的初始化
      try
      {
        string json = File.ReadAllText("config.json");
        Config config = System.Text.Json.JsonSerializer.Deserialize<Config>(json);
        MainWModel.Ip = config.ModbusTcpIP;
        MainWModel.Port = config.ModbusTcpPort.ToString();
        MainWModel.LogPath = config.LogPath;

        MainWModel.RecipeNumberAddress = config.RecipeNumberAddress;
        MainWModel.DisplayOrderNumberAddress = config.DisplayOrderNumberAddress;
        MainWModel.WriteNumberAddress = config.WriteNumberAddress;
        MainWModel.StartIngredientAddress = config.StartIngredientAddress;
        MainWModel.IngredientCompletionAddress = config.IngredientCompletionAddress;
        MainWModel.DeviceStatusAddress = config.DeviceStatusAddress;
        MainWModel.InformationErrorAddress = config.InformationErrorAddress;

        MainWModel.ReadDataLength = config.ReadDataLength;
        MainWModel.WriteDataLength = config.WriteDataLength;

        DataType = dataTypes.First(m => m.ID == config.RecipeDataID);//绑定到下拉框显示的值
        DataFormulas = formulas.First(m => m.FormatID == config.DataFormulasID);

        OrderNumberType = dataTypes.First(m => m.ID == config.OrderNumberTypeID);//绑定到下拉框显示的值
        OrderNumberData = formulas.First(m => m.FormatID == config.OrderNumberDataFormat);
      }
      catch (Exception ex)
      {
        MainWModel.ShowMessage("初始化参数加载失败，" + ex.Message, false);
        //return;
      }
    }

    /// <summary>
    /// 初始数据类型
    /// </summary>
    private void SetRecipeNumberTypeData()
    {
      dataTypes.Clear();
      int i = 0;
      foreach (DirectionType type in Enum.GetValues(typeof(DirectionType)))
      {
        dataTypes.Add(new RecipeNumberDataType
        {
          ID = i,
          DataName = type.ToString()
        });
        i++;
      }
    }

    /// <summary>
    /// 初始数据格式
    /// </summary>
    private void SetFormulaNumberDataFormat()
    {
      formulas.Clear();
      int i = 0;
      foreach (BPADataFormat item in Enum.GetValues(typeof(BPADataFormat)))
      {
        formulas.Add(new FormulaNumberDataFormat
        {
          FormatID = i,
          DataFormatName = item.ToString()
        });
        i++;
      }
    }

    /// <summary>
    /// 初始化配方数据
    /// </summary>
    private void GetFormulaData()
    {
      try
      {
        formulaAttachment.Clear();//先清除集合
        string jsons = File.ReadAllText("Formula.json");
        FormulaAttList configs = System.Text.Json.JsonSerializer.Deserialize<FormulaAttList>(jsons);
        configs.FormulaList.ForEach(s =>
        {
          formulaAttachment.Add(new FormulaAttachment
          {
            FormulaName = s.FormulaName,
            FormulaNumber = s.FormulaNumber,
            SerialNumber = s.SerialNumber,
            spare = s.spare,
          });
        });
      }
      catch (Exception ex)
      {

        MainWModel.ShowMessage("初始化配方数据加载失败，" + ex.Message, false);
      }

    }

    /// <summary>
    /// 打开配料附件配置文件
    /// </summary>
    /// <param name="text"></param>
    private void OpenIngredientAttachment(object text)
    {
      //D:\黑波罗科技\BPA.WeimoCube\BPA.WeimoCube\bin\Debug\net6.0-windows
      string PeiLaoPath = System.IO.Directory.GetCurrentDirectory();
      if (Directory.Exists(PeiLaoPath))
      {
        Process.Start("explorer.exe", PeiLaoPath);
      }
    }

    /// <summary>
    /// 打开日志文件
    /// </summary>
    private void OpenLogFolder(object text)
    {
      if (Directory.Exists(MainWModel.LogPath))
      {
        Process.Start("explorer.exe", MainWModel.LogPath);
      }
    }

    /// <summary>
    /// 打开文件夹
    /// </summary>
    /// <param name="text"></param>
    private void LogPathSelect(object text)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      //过滤文件类型
      //openFileDialog.Filter = "文档(*.txt)|*.txt|所有文件(*.*)|*.*";
      //允许多选
      openFileDialog.Multiselect = true;
      if (openFileDialog.ShowDialog() == true)
      {
        string paths = Path.GetDirectoryName(openFileDialog.FileName);
        //FileName 完整路径：相对路径+文件名+后缀
        //文件内容读取
        MainWModel.LogPath = paths;
        LogProvider.LogPath = MainWModel.LogPath;
        //string[] names = openFileDialog.FileNames;
      }

    }

    /// <summary>
    /// 点击设置
    /// </summary>
    /// <param name="text"></param>
    private void Config(object text)
    {
      //MainWModel.IsStart = false;
      MainWModel.BlurRadius = 10;
    }
    /// <summary>
    /// 隐藏设置界面
    /// </summary>
    private void ConfigClose(object text)
    {
      MainWModel.BlurRadius = 0;

      // 数据保存的动作
      try
      {
        Config config = new Config();
        config.ModbusTcpIP = MainWModel.Ip;
        config.ModbusTcpPort = Convert.ToInt32(MainWModel.Port);
        config.LogPath = MainWModel.LogPath;
        config.RecipeNumberAddress = MainWModel.RecipeNumberAddress;
        config.DisplayOrderNumberAddress = MainWModel.DisplayOrderNumberAddress;
        config.WriteNumberAddress = MainWModel.WriteNumberAddress;
        config.StartIngredientAddress = MainWModel.StartIngredientAddress;
        config.IngredientCompletionAddress = MainWModel.IngredientCompletionAddress;
        config.DeviceStatusAddress = MainWModel.DeviceStatusAddress;
        config.InformationErrorAddress = MainWModel.InformationErrorAddress;
        config.RecipeDataID = DataType.ID;
        config.DataFormulasID = DataFormulas.FormatID;
        config.OrderNumberTypeID = OrderNumberType.ID;
        config.OrderNumberDataFormat = OrderNumberData.FormatID;

        config.ReadDataLength = MainWModel.ReadDataLength;
        config.WriteDataLength= MainWModel.WriteDataLength;
        // Framework   Nuget:安装 Newtonsoft.Json  库，参照472版本的程序
        var json = System.Text.Json.JsonSerializer.Serialize(config);
        File.WriteAllText("config.json", json);

        LogProvider.WriteLog("配置文件修改完成", MainWModel.IsLog);
      }
      catch (Exception ex)
      {
        LogProvider.WriteLog("写配置文件失败!" + ex.Message, MainWModel.IsLog, isError: true);
        MainWModel.ShowMessage("写配置文件失败!" + ex.Message, false);
      }

    }

    /// <summary>
    /// 实现 连接modbusTcp 按钮事件
    /// </summary>
    /// <param name="text"></param>
    private void DoConnectModbusTcpCommand(object text)
    {

      //if (TiJiaoPanDuan())
      //{
      //  modbus.ConnectModbusTcp(MainWModel.Ip, Convert.ToInt32(MainWModel.Port));
      //  MainWModel.TCPDRWIsEnabled = true;
      //}
    }

    /// <summary>
    /// 开启监听
    /// </summary>
    private void StartMointor()
    {

      bool connctionSoss = false;
      var task = Task.Factory.StartNew(async () =>
      {
        while (!cts.IsCancellationRequested)
        {
          //Debug.WriteLine("开启监听");
          var state = new CommcationLibray.Result();
          await Task.Delay(400);
          if (!MainWModel.IsStart) continue;
          if (MainWModel.ConnectionStatus) continue;
          if (TiJiaoPanDuan())
          {
            try
            {
              if (!connctionSoss)
                MainWModel.ShowMessage("连接中.......");

              Application.Current.Dispatcher.Invoke(new Action(async () =>
              {
                state = await modbus.ConnectModbusTcp(MainWModel.Ip, Convert.ToInt32(MainWModel.Port));
              }), System.Windows.Threading.DispatcherPriority.SystemIdle, null);


              await Task.Delay(500);

              if (!state.State)
              {
                MainWModel.ShowMessage("连接失败", false);
                MainWModel.ConnectionStatus = state.State;
                connctionSoss = false;
              }
              else
              {
                connctionSoss = true;
                MainWModel.ConnectionStatus = state.State;
                MainWModel.ShowMessage("连接成功");
                this.GetInformationError();
                this.GetDeviceStatus();
              }

            }
            catch (Exception ex)
            {
              MessageError($"连接失败!!! 失败信息：{ex.Message}");
            }
          }
        }
      }, cts.Token);
      tasks.Add(task);
    }


    /// <summary>
    /// 点击了取消配料,该标志位为True.上位机收到信息后主动置False
    /// </summary>
    private void GetInformationError()
    {
      var task = Task.Factory.StartNew(async () =>
      {
        while (!cts.IsCancellationRequested)
        {
          await Task.Delay(400);
          try
          {
            if (!IsState()) return;
            var Result = new CommcationLibray.Result();
            Result = modbus.ReadCancelIngredients(MainWModel.InformationErrorAddress);
            if (!Result.State)
            {
              MessageError($"获取取消配料数据失败!!! 失败信息：{Result.Exception}");
            }
            else
            {
              //为1说明点击了取消配料
              if ((bool)Result.Datas)
              {
                this.WhetherCancel = true;
                this.WriteInformationError();
                MainWModel.OrderPushState = $"订单->{MainWModel.CurrentOrderNumber}已取消!";
                MessageBox.Show($"当前订单-> {MainWModel.CurrentOrderNumber} 已取消!");
              }
            }
          }
          catch (Exception ex)
          {
            MessageError($"获取取消配料数据失败!!! 失败信息：{ex.Message}");
          }
        }
      }, cts.Token);
      tasks.Add(task);
    }

    /// <summary>
    /// 循环获取设备状态
    /// </summary>
    private void GetDeviceStatus()
    {
      var task = Task.Factory.StartNew(async () =>
      {
        while (!cts.IsCancellationRequested)
        {
          await Task.Delay(400);
          try
          {
            if (!IsState()) return;
            var Result = new CommcationLibray.Result();
            Result = modbus.ReadDeviceStatus(MainWModel.DeviceStatusAddress);
            if (!Result.State)
            {
              MessageError($"获取设备状态数据失败!!! 失败信息：{Result.Exception}");
            }
            else
            {
              this.DeviceStatus = (bool)Result.Datas;
              this.StateFanKui((bool)Result.Datas);
            }
          }
          catch (Exception ex)
          {
            MessageError($"获取设备状态数据失败!!! 失败信息：{ex.Message}");
          }
        }
      }, cts.Token);
      tasks.Add(task);
    }


    /// <summary>
    /// 点击取消配料时  上位机自动触发 设置值为false
    /// </summary>
    private void WriteInformationError()
    {
      if (!IsState()) return;
      var Result = new CommcationLibray.Result();
      Result = modbus.WriteCancelIngredients(MainWModel.InformationErrorAddress, false);
      if (!Result.State)
      {
        MessageError($"写入读取配料状态失败!!! 失败信息：{Result.Exception}");
      }
    }


    /// <summary>
    /// 封装读取操作
    /// </summary>
    /// <param></param>
    private void ReadCaoZuo()
    {
      try
      {
        switch (OrderNumberType.ID)
        {
          case 0:
            ReadOrders<byte>();
            break;
          case 1:
            ReadOrders<int>();
            break;
          case 2:
            ReadOrders<uint>();
            break;
          case 3:
            ReadOrders<short>();
            break;
          case 4:
            ReadOrders<ushort>();
            break;
          case 5:
            ReadOrders<float>();
            break;
          case 6:
            ReadOrders<string>();
            break;
          case 7:
            ReadOrders<bool>();
            break;
          case 8:
            ReadOrders<double>();
            break;
        }
      }
      catch (Exception ex)
      {
        MessageError($"数据转换失败!!! 失败信息：{ex.Message}");
      }
    }

    /// <summary>
    /// 读取订单
    /// </summary>
    private void ReadOrders<T>()
    {
      var Result = new CommcationLibray.Result();
      Result = modbus.GetOrders<T>(MainWModel.DisplayOrderNumberAddress,MainWModel.ReadDataLength, (BPADataFormat)OrderNumberData.FormatID );//BPADataFormat.CDAB
      if (!Result.State)
      {
        MessageError($"读取订单失败!!! 失败信息：{Result.Exception}");
      }
      else
      {
        MainWModel.ShowMessage("读取订单成功");
        MainWModel.CurrentOrderNumber = Result.Datas ;
      }

    }

    /// <summary>
    /// 写入编号
    /// </summary>
    /// <param name="test"></param>
    private void WriteNumber(object test)
    {
      WriteCaoZuo(0);

    }
    /// <summary>
    /// 确定写入编号
    /// </summary>
    /// <param name="test"></param>
    private void DetermineWriteNumber(object test)
    {
      WriteCaoZuo(1);
    }

    /// <summary>
    /// 封装写入操作
    /// </summary>
    /// <param name="type"> 0 表示写入 ，1表示确定写入</param>
    private void WriteCaoZuo(int type)
    {
      try
      {
        if (MainWModel.FormulaNumber == null)
        {
          MessageError($"请输入配方编号!!!");
          return;
        }
        switch (DataType.ID)
        {
          case 0:
            WriteCaoZuoDataType(type, Convert.ToByte(MainWModel.FormulaNumber));
            break;
          case 1:
            WriteCaoZuoDataType(type, Convert.ToInt32(MainWModel.FormulaNumber));
            break;
          case 2:
            WriteCaoZuoDataType(type, Convert.ToUInt32(MainWModel.FormulaNumber));
            break;
          case 3:
            WriteCaoZuoDataType(type, (short)Convert.ToInt32(MainWModel.FormulaNumber));
            break;
          case 4:
            WriteCaoZuoDataType(type, (ushort)Convert.ToInt32(MainWModel.FormulaNumber));
            break;
          case 5:
            WriteCaoZuoDataType(type, (float)Convert.ToDouble(MainWModel.FormulaNumber));
            break;
          case 6:
            WriteCaoZuoDataType(type, Convert.ToString(MainWModel.FormulaNumber));
            break;
          case 7:
            WriteCaoZuoDataType(type, Convert.ToBoolean(Convert.ToInt32(MainWModel.FormulaNumber)));
            break;
          case 8:
            WriteCaoZuoDataType(type, Convert.ToDouble(MainWModel.FormulaNumber));
            break;
        }
      }
      catch (Exception ex)
      {
        MessageError($"数据转换失败!!! 失败信息：{ex.Message}");
      }
    }


    private void WriteCaoZuoDataType<T>(int type, T values)
    {
      if (!IsState()) return;
      if (!this.DetermineWriteStatus && !this.DeviceStatus)
      {
        BPADataFormat bPAData = (BPADataFormat)DataFormulas.FormatID;
        var Result = new CommcationLibray.Result();
        string error = type == 0 ? "写入" : "确定写入";
        Result = type == 0 ? modbus.WriteRecipeNumber<T>(MainWModel.RecipeNumberAddress, values, MainWModel.WriteDataLength, (BPADataFormat)DataFormulas.FormatID)
                   : modbus.DetermineWriteNumberCommand(MainWModel.WriteNumberAddress, true);
        if (!Result.State)
        {
          MessageError($"{error}编号失败!!! 失败信息：{Result.Exception}");
        }
        else
        {
          MainWModel.ShowMessage($"{error}编号成功");
          if (type == 0)
            this.ReadCaoZuo();
          else
          {
            this.DetermineWriteStatus = true;
          }
        }
      }
      else
      {
        MessageBox.Show("请等待上次订单完成后再操作!!!");
      }
    }

    /// <summary>
    /// 启动配料
    /// </summary>
    /// <param name="test"></param>
    private void StartingIngredients(object test)
    {

    }

    private void StateFanKui(bool state)
    {
      switch (state)
      {
        case false:
          MainWModel.TopDianState = "空闲中......";
          if (this.DetermineWriteStatus && !this.WhetherCancel)
          {
            MainWModel.OrderPushState = "已完成";
          }
          this.DetermineWriteStatus = false;
          this.WhetherCancel = false;
          break;
        case true:
          MainWModel.OrderPushState = "已推送";
          if (this.DetermineWriteStatus)
          {
            MainWModel.TopDianState = "待操作";
          }
          break;
        default:
          break;
      }
    }

    /// <summary>
    /// 错误信息
    /// </summary>
    /// <param name="error"></param>
    private void MessageError(string error)
    {
      MainWModel.ShowMessage(error, false);
      LogProvider.WriteLog(error, MainWModel.IsLog, isError: true);
    }

    private bool TiJiaoPanDuan()
    {
      if (string.IsNullOrEmpty(MainWModel.Ip))
      {
        MainWModel.ShowMessage("请输入IP地址!!!", false);
        return false;
      }
      if (string.IsNullOrEmpty(MainWModel.Port))
      {
        MainWModel.ShowMessage("请输入端口号!!!", false);

        return false;
      }
      if (!PanDuanGongJ.IsIPGf(MainWModel.Ip))
      {
        MainWModel.ShowMessage("请输入合法的IP地址!!!", false);

        return false;
      }
      if (!PanDuanGongJ.IsIPPort(MainWModel.Port))
      {
        MainWModel.ShowMessage("请输入正确的端口号!!!", false);

        return false;
      }
      return true;
    }

    /// <summary>
    /// 判断是否打开监听
    /// </summary>
    /// <returns></returns>
    private bool IsState()
    {
      if (!MainWModel.IsStart || !MainWModel.ConnectionStatus)
      {
        MainWModel.ShowMessage("未连接状态!!!", false);
        return false;
      }
      else
      {
        return true;
      }
    }
    /// <summary>
    /// 退出
    /// </summary>
    /// <param name="text"></param>
    private void Close(object text)
    {
      CancelAllTasksAsync();
    }

    public void CancelAllTasksAsync()
    {
      cts?.Cancel();
    }
  }
}



